/* DEC Rainbow 100 Low-Level Screen Output Routines
 *
 * Copyright (c) 2011 Jeffrey Armstrong <jeff@rainbow-100.com>
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

/* NOTE: Much of the original code using direct RAM access is still
 *       present, but commented.  It may still be of interest to other
 *       Rainbow developers.
 */

#include <stdio.h>
#include <dos.h>

#define R       _scr_row
#define C       _scr_col

int _scr_col;
int _scr_row;

char _rba;

int _vrow[] = {0x12,0x99,0x120,0x1A7,0x22E,0x2B5,
                  0x33C,0x3C3,0x44A,0x4D1,0x558,0x5DF,
                  0x666,0x6ED,0x774,0x7FB,0x882,0x909,
                  0x990,0xA17,0xA9E,0xB25,0xBAC,0xC33,0xCBA};

/* No attributes = 2+4+8 = 14 */
#define ATTR_NORMAL     14

int _rb_scroll_top, _rb_scroll_bottom;

void rbs_setcursor(int row, int col, int enable);

void rbs_init()
{
        R = 0;
        C = 0;
        _rba = (char)ATTR_NORMAL;
        
        _rb_scroll_top=0; 
        _rb_scroll_bottom=24;

        printf("%c[?3l",033);
}

void rbs_move(int row, int col)
{
        R = row;
        C = col;
}

void rbs_seta(char a)
{
        _rba = a;               
}

void rbs_putc(char c)
{
static char near *dest = NULL;
static struct REGPACK r;
struct SREGS segregs;

        if(R < 0 || R > 24 || C < 0 || C > 79) return;

        /* dest = MK_FP(0xEE00, (unsigned int)_vrow[R] + (unsigned int)C);
         * dest[0] = c;
         * dest = MK_FP(0xEF00, (unsigned int)_vrow[R] + (unsigned int)C);
         * dest[0] = _rba;
         */

        segread(&segregs);

        if(dest == NULL)
            dest = (char near *)malloc(2);

        dest[0] = c; dest[1] = _rba;       
 
        r.r_di = (unsigned int)0x14;
        r.r_ax = (unsigned int)0;
        r.r_bx = (unsigned int)(C+1)*(unsigned int)256 + (unsigned int)(R+1); 
        r.r_cx = (unsigned int)1;
        r.r_dx = (void near *)(dest+1);
        r.r_si = (void near *)dest;
        r.r_bp = segregs.ds;
        
        intr(0x18,&r);

}

void rbs_putc_broke(char c)
{
char near *dest;
static struct REGPACK r;
union REGS ru;
char c_l, a_l;

        if(R < 0 || R > 24 || C < 0 || C > 79) return;

        /* dest = MK_FP(0xEE00, (unsigned int)_vrow[R] + (unsigned int)C);
         * dest[0] = c;
         * dest = MK_FP(0xEF00, (unsigned int)_vrow[R] + (unsigned int)C);
         * dest[0] = _rba;
         */

        rbs_setcursor(R,C,0);
        /* printf("%c",c); */
        ru.di = (unsigned int)0;
        ru.h.al = c;
        int86(0x18,&ru,&ru);
        return;
        
        c_l = c; a_l = _rba;

        r.r_di = (unsigned int)0x14;
        r.r_ax = (unsigned int)2;
        r.r_bx = (unsigned int)(C*256 + R); 
        r.r_cx = (unsigned int)1;
        r.r_dx = FP_OFF(&a_l);
        r.r_si = FP_OFF(&c_l);
        r.r_bp = FP_SEG(&c_l);
        
        intr(0x18,&r);

}

void rbs_clearrow(int r)
{
        if(r<0 || r > 24) return;

        rbs_setcursor(r,1,0);
        printf("%c[2K");

        /* memset(MK_FP(0xEE00, (unsigned int)_vrow[r]),0,80*sizeof(char));
         * memset(MK_FP(0xEF00, (unsigned int)_vrow[r]),(char)ATTR_NORMAL,80*sizeof(char));
         */
}

void rbs_clearscreen()
{
int i;
        printf("%c[2J",033);
        /* printf("%c[?3l",033);
         * for(i=0;i<24;i++)
         *        rbs_clearrow(i);
         */
}

void rbs_scrollup()
{
int i;

        printf("%cE",033);

        /* for(i=1+_rb_scroll_top;i<=_rb_scroll_bottom;i++) {
         *        memcpy(MK_FP(0xEE00, (unsigned int)_vrow[i-1]),
         *                MK_FP(0xEE00, (unsigned int)_vrow[i]),
         *                80*sizeof(char));
         *         memcpy(MK_FP(0xEF00, (unsigned int)_vrow[i-1]),
         *                MK_FP(0xEF00, (unsigned int)_vrow[i]),
         *                80*sizeof(char));
         * }
         * rbs_clearrow(_rb_scroll_bottom);
         */
}

void rbs_scrolldown()
{
int i;

        printf("%cE",033);

        /* for(i=_rb_scroll_bottom;i>_rb_scroll_top;i--) {
         *         memcpy(MK_FP(0xEE00, (unsigned int)_vrow[i]),
         *                MK_FP(0xEE00, (unsigned int)_vrow[i-1]),
         *                80*sizeof(char));
         *         memcpy(MK_FP(0xEF00, (unsigned int)_vrow[i]),
         *                MK_FP(0xEF00, (unsigned int)_vrow[i-1]),
         *                80*sizeof(char));
         * }
         * rbs_clearrow(_rb_scroll_top);
         */
}

void rbs_setscrollregion(int top, int bottom)
{
        _rb_scroll_top = top;
        _rb_scroll_bottom = bottom;
}

void rbs_getscrollregion(int *top, int *bottom)
{
        *top = _rb_scroll_top;
        *bottom = _rb_scroll_bottom;
}

void rbs_getcursor(int *row, int *col)
{
char *dest;

       dest = MK_FP(0xEE00, (unsigned int)0xF41);
       *row = (int)dest[1];
       *col = (int)dest[0];
}

void rbs_setcursor(int row, int col, int enable)
{
char *dest;
union REGS r;
char epos[16];
int i;

        if(row < 0 || row > 24 || col < 0 || col > 131) return;
        
        sprintf(epos,"%c[%3d;%3df",(char)27,row,col);
        for(i=2;i<9;i++) {
                if(epos[i] == ' ')
                        epos[i] = '0';
        }
        printf(epos);        

}
